@@ -208,7 +208,14 @@ clutter_stage_x11_get_geometry (ClutterStageWindow    *stage_window,
 
   geometry->x = geometry->y = 0;
 
-  /* If we're fullscreen, return the size of the display. */
+  /* If we're fullscreen, return the size of the display.
+   *
+   * FIXME - this is utterly broken for anything that is not a single
+   * head set up; the window manager will give us the right size in a
+   * ConfigureNotify, but between the fullscreen signal emission on the
+   * stage and the following frame, the size returned by the stage will
+   * be wrong.
+   */
   if (_clutter_stage_is_fullscreen (stage_cogl->wrapper) &&
       stage_x11->fullscreening)
     {
@@ -531,6 +538,37 @@ clutter_stage_x11_set_fullscreen (ClutterStageWindow *stage_window,
    * queue a relayout etc. */
 }
 
+void
+_clutter_stage_x11_events_device_changed (ClutterStageX11 *stage_x11,
+                                          ClutterInputDevice *device,
+                                          ClutterDeviceManager *device_manager)
+{
+  ClutterStageCogl *stage_cogl = CLUTTER_STAGE_COGL (stage_x11);
+  int event_flags = 0;
+
+  if (clutter_input_device_get_device_mode (device) == CLUTTER_INPUT_MODE_FLOATING)
+    event_flags = CLUTTER_STAGE_X11_EVENT_MASK;
+
+  _clutter_device_manager_select_stage_events (device_manager,
+                                               stage_cogl->wrapper,
+                                               event_flags);
+}
+
+static void
+stage_events_device_added (ClutterDeviceManager *device_manager,
+                           ClutterInputDevice *device,
+                           gpointer user_data)
+{
+  ClutterStageWindow *stage_window = user_data;
+  ClutterStageCogl *stage_cogl = CLUTTER_STAGE_COGL (stage_window);
+  int event_flags = CLUTTER_STAGE_X11_EVENT_MASK;
+
+  if (clutter_input_device_get_device_mode (device) == CLUTTER_INPUT_MODE_FLOATING)
+    _clutter_device_manager_select_stage_events (device_manager,
+                                                 stage_cogl->wrapper,
+                                                 event_flags);
+}
+
 static gboolean
 clutter_stage_x11_realize (ClutterStageWindow *stage_window)
 {
@@ -614,9 +652,16 @@ clutter_stage_x11_realize (ClutterStageWindow *stage_window)
    * the event mask we passed to XSelectInput as the template
    */
   device_manager = clutter_device_manager_get_default ();
-  _clutter_device_manager_select_stage_events (device_manager,
-                                               stage_cogl->wrapper,
-                                               event_flags);
+  if (G_UNLIKELY (device_manager != NULL))
+    {
+      _clutter_device_manager_select_stage_events (device_manager,
+                                                   stage_cogl->wrapper,
+                                                   event_flags);
+
+      g_signal_connect (device_manager, "device-added",
+                        G_CALLBACK (stage_events_device_added),
+                        stage_window);
+    }
 
   clutter_stage_x11_fix_window_size (stage_x11,
                                      stage_x11->xwin_width,
